/***************************************************************
	Roboscopio.c
   Autor: Ulises Bigliati.
   ---------------------------------------------------
   version	inicio		finalizacin	comentarios
   1.0    	01/06/08		16/07/08       Interfaz Skymap
   ---------------------------------------------------
   1.1		19/07/08	   31/07/08      	Interfaz web
   Detalles:
   			-Interfaz Web de monitoreo y control.
            -Funcin demo "fast auto-track".
            -Funcin de demostracin de movimientos.
   			-Modificacin menor en time-out de teclado.
            -Correccin de bug en altaz-radec.lib (Alt-az-> RA-Dec).
            -Reduccin de velocidad de motores (menor inercia)
	----------------------------------------------------
   1.11		29/12/08   						Volcado a nota de aplicacin
   Detalles:
   			-Creacin de mdulo iweb.lib: encapsular interfaz web.
            -Creacin de mdulo command.lib: agrupar comandos del telescopio.
            -Creacin de mdulo joystick.lib: agrupar funciones y variables del joystick.
            -Cambio en estructura de archivos del proyecto

/**************************************************************/
#class auto
#memmap xmem

#define DS1 6		//led, port F bit 6
#define DS2 7		//led, port F bit 7
#define S1  4		//switch, port F bit 4
#define S2  7		//switch, port B bit 7

#define MoveAlt(int) motor_u_move(int)
#define MoveAz(int) motor_b_move(int)

#use motors.lib  			//Step to Step motor library
#use juliandate.lib  	//Julian day caltulation library
#use altaz-radec.lib   	//Coordenates conversion library
#use const.lib   			//Constants for communication that are shared with Skymap Roboscope driver
#use keyboard.lib   		//Joystick control library
#use sensor.lib    		//Sensor defines, macros & init functions
#use calibrate.lib      //Calibration & initialization functions
#use iweb.lib				//Encapsulates web interface configuration


//The input and output serial buffers sizes.
#define DINBUFSIZE  63
#define DOUTBUFSIZE 63
#define SBUFMAXLEN 32
#define STIMEOUT 3000UL // time out 3 seconds after receiving a character unless cof_serDread completes

#define LED	5 //In Port D
#define LEDOFF BitWrPortI(PDDR, &PDDRShadow, 0,LED)
#define LEDON 	BitWrPortI(PDDR, &PDDRShadow, 1,LED)

#use command.lib	//Fundamental telescope functions and command processor
#use joystick.lib	//manual command control


/***********************************
	Operational variables
************************************/
static int az_steps;  			//Hold quantity of total azimuth steps (for complete 360)
static int alt_steps; 			//Hold quantity of total altitud steps (for complete 90)
struct stPosition myPos;			//Local Latitud, Longitud and Time difference

static struct stAltAz myRes;  //Resolution for both axis (360/az_steps and 90/alt_steps)
static struct stAltAz residuo;//decimal residual points result of GOTO function after an auto-track calling.

shared struct stRAdec curRAdec; 	//current positioning of telescope in RADec coordenates
shared struct stRAdec newRAdec; 	//auxiliary RADec while telescope position is changing

shared struct stAltAz newAltAz;   //auxiliary alt-azimut while telescope position is changing
shared struct stAltAz curAltAz;   //current positioning of telescope in alt-azimut coordenates

struct tm   rtc;     				//Global Real Time structure

void main()
{
	int getOk, done;
	char buffer[SBUFMAXLEN + 1];
   char cKey;

//VARIABLES INITIALIZATION
	flag_joystick_enabled = flag_sync_enabled = done = 0;	//initialize flags to false value
	auto_track_mode = AUTOTRACK_STOP; 					//normal initialization mode.
   AutoTrackInterval = AUTOTRACK_NORMAL_INTERVAL; // normal initialization mode.

//INITIALIZATION FUNCTIONS
   brdInit();  		//initialize board
   motor_u_init();   //init unipolar motor, used for Altitud operation
  	motor_b_init();   //init bipolar motor, used for Azimuth operation
  	sensor_init();		//init sensor ports
	keyb_init(); 		//init keyboard matrix

   BitWrPortI(PDDDR, &PDDDRShadow, 1,LED);//setup LED
   BitWrPortI(PDDR, &PDDRShadow, 0,LED);

/*****************************************************
 * Star Up Telescope
 *****************************************************/
 //if S1 is pressed then calibrate
   if (!BitRdPortI(PFDR, S1))
   {
	   BitWrPortI(PFDR, &PFDRShadow, 0, DS1);
      if( calibrate(&az_steps, &alt_steps) ){
      //Save calibration data
      	writeUserBlock(sizeof(myPos), &az_steps, sizeof(az_steps));
			writeUserBlock(sizeof(az_steps)+sizeof(myPos), &alt_steps, sizeof(alt_steps));
         //printf("Pasos en azimuth: %d\nPasos en altitud: %d\n", az_steps, alt_steps);
      }
	   BitWrPortI(PFDR, &PFDRShadow, 1, DS1);
   }
 //Restore calibration data
   readUserBlock(&myPos, 0,	sizeof(myPos));
   readUserBlock(&az_steps, 	sizeof(myPos), sizeof(az_steps));
  	readUserBlock(&alt_steps, 	sizeof(az_steps)+sizeof(myPos), sizeof(alt_steps));

 //Calculate resolution for steppers motors
   myRes.az =  360.0 / az_steps;
   myRes.alt=  90.0 / alt_steps;
 //Initilizes residual values for motion correction.
   residuo.alt=0.0;
   residuo.az=0.0;

 //if S2 is pressed then demostrate!
   if (!BitRdPortI(PBDR, S2))
   {
	   BitWrPortI(PFDR, &PFDRShadow, 0, DS2);
		init_demo();
	   BitWrPortI(PFDR, &PFDRShadow, 1, DS2);
   }
  //Initilizes the initial position for Altitud=0 and Az=0
  	if ( init_position() ) {
  	   curAltAz.alt = 0.0;
      curAltAz.az = 0.0;
  	}
  //Turn Start Altitude & Azimuth position into RA-Dec coordenates.
 	tm_rd(&rtc);
	altaz2radec( &rtc,	&curRAdec, &curAltAz, &myPos);

	iweb_init();

   serDopen(38400);

   while (!done) {
      http_handler();//Web server attention...
		loophead();// for proper cofunction handling...
      /*************************************************************************
       * Command interpreter costate.
       ************************************************************************/
   	costate
      {  // yields until getting a null terminated string
   		wfd getOk = cof_serDgets(buffer, SBUFMAXLEN, STIMEOUT);
   		if (getOk)
         { //go to command interface
           	printf("%s\r\n",buffer);
			  	cmd_process(buffer);		//process the command and return with reply
			  	wfd cof_serDputs(buffer);// Replys the response
           	serDrdFlush();				//clear rest of previous command
           	getOk=0;
           	printf("%s\r\n",buffer);
      	}
      	else{ //Time Out
      		wfd cof_serDputs("0");
      	}
      }
      /*************************************************************************
       * Auto-track Costates
       ************************************************************************/
 		costate
		{ //Peridical Auto-track calling for last RA-Dec coordenate.
			if (auto_track_mode)//activated with SLEW, S2 switch or Web command
			{  //Mode is 1 normal, 2 fast (demo), 0 stopped.
           	waitfor(DelaySec(AutoTrackInterval));
            	auto_track_function(auto_track_mode);
         }
         else
         { //set new RA-Dec if auto-track is OFF (the earth continues moving...)
         	tm_rd(&rtc);
	         altaz2radec(   &rtc, &curRAdec, &curAltAz,   &myPos);
         }
		}
		costate
		{ //Board Switch for to activate / deactivate auto-track function.
			if (BitRdPortI(PBDR, S2))		//wait for switch S2 press
				abort;
			waitfor(DelayMs(50));			//switch press detected if got to here
			if (BitRdPortI(PBDR, S2))		//wait for switch release
				auto_track_mode = ! auto_track_mode;
		}
      costate
		{	// Blink Led if auto-track is activated
			if (auto_track_mode)
         {
				waitfor(DelayMs( 600/(1+auto_track_mode) ) );
         	BitWrPortI(PFDR, &PFDRShadow, !(PFDRShadow & (1<<DS2)), DS2);
            BitWrPortI(PDDR, &PDDRShadow, !(PDDRShadow & (1<<DS2)) ,LED);
         }
      }
      /***********************************************************************
       * Joystick control Costates
       **********************************************************************/
      costate
		{  // Look for Joystick Key pressed into keyboard matrix
			if (  !( cKey = keyb_get_key() ) )
         	abort;
         LEDON;
         waitfor(DelayMs(200));
         joystick_action(cKey);
         cKey=0;
         LEDOFF;
      }
 		costate
		{  //Check if manual command finished by time-out (save motors life)
			if (flag_joystick_enabled)
         {
           	waitfor(DelaySec(1));
            if( !(--keyb_tout.v) )
                motor_u_stop();
            if( !(--keyb_tout.h) )
                motor_b_stop();
         }
		}
      costate
		{	// Blink Led if Joystick is still used
			if (flag_joystick_enabled)
         {
				waitfor(DelayMs(500));
         	BitWrPortI(PFDR, &PFDRShadow, !(PFDRShadow & (1<<DS1)) , DS1);
         }
      }
      /*************************************************
       *  Switch S1 ON/OFF functionality :
       *  	-Joystick control
       *		-Fast Auto-Track function
       *************************************************/
		costate
		{
			if (BitRdPortI(PFDR, S1))		//wait for switch S1 press
				abort;
			waitfor(DelayMs(50));			//switch press detected if got to here
			if (BitRdPortI(PFDR, S1))
			{  //Check if manual command finished by switch control
         	if(flag_joystick_enabled)
            	joystick_action(255);
            else
               auto_track_fast_OnOff();
         }
		}
      costate
		{  //update string time variables to show the time in ascci format (web)
      	waitfor(DelaySec(2));
         if ( auto_track_mode == AUTOTRACK_NORMAL || auto_track_mode == AUTOTRACK_STOP)
				tm_rd(&rtc);// get date time normally;
         get_astro_time();//call times function;
		}
   }
   while (serDwrFree() != DOUTBUFSIZE) ;      // allow transmission to complete before closing
   serDclose();
}


